import logging
import os

import aiohttp

from ose_plugin_hbcp.search_api.APIService import APIService
from .AddictOVocabClient import AddictOVocabClient
from ose.services.ConfigurationService import ConfigurationService

PROP_ADDICTO_VOCAB_API_PATH = "ADDICTO_VOCAB_API_PATH"
PROP_ADDICTO_VOCAB_API_AUTH_TOKEN = "ADDICTO_VOCAB_API_AUTH_TOKEN"


class AddictOVocabService(APIService[AddictOVocabClient]):
    _logger = logging.getLogger(__name__)
    
    @property
    def repository_name(self) -> str:
        return "AddictO"

    def __init__(self, config: ConfigurationService, session: aiohttp.ClientSession):
        path = config.app_config.get(PROP_ADDICTO_VOCAB_API_PATH, os.environ.get(PROP_ADDICTO_VOCAB_API_PATH))
        auth_token = config.app_config.get(
            PROP_ADDICTO_VOCAB_API_AUTH_TOKEN, os.environ.get(PROP_ADDICTO_VOCAB_API_AUTH_TOKEN, None)
        )

        api_client = AddictOVocabClient(
            path, session, auth_token, config.app_config.get("ENVIRONMENT", "debug") != "production"
        )

        super().__init__(config, api_client)



from typing import Literal, Dict, Tuple

from aiohttp import ClientSession

from ose_plugin_hbcp.search_api.APIClient import APIClient
from ose.model.Term import Term
from ose.model.TermIdentifier import TermIdentifier


class AddictOVocabClient(APIClient):
    _session: ClientSession

    _term_link_to_relation_mapping: Dict[
        str, Tuple[TermIdentifier, Literal["single", "multiple", "multiple-per-line"]]] = {
        "synonyms": (TermIdentifier(id="IAO:0000118", label="alternative label"), "multiple"),
        "definition": (TermIdentifier(id="IAO:0000115", label="definition"), "single"),
        "informalDefinition": (TermIdentifier(label="informalDefinition"), "single"),
        "addictoSubOntology": (TermIdentifier(label="lowerLevelOntology"), "multiple"),
        "curatorNote": (TermIdentifier(id="IAO:0000232", label="curator note"), "single"),
        "curationStatus": (TermIdentifier(id="IAO:0000114", label="has curation status"), "single"),
        "comment": (TermIdentifier(id="rdfs:comment", label="rdfs:comment"), "single"),
        "examples": (TermIdentifier(id="IAO:0000112", label="example of usage"), "multiple-per-line"),
        "fuzzySet": (TermIdentifier(label="fuzzySet"), "single"),
        "fuzzyExplanation": (TermIdentifier(label="fuzzyExplanation"), "single"),
        "crossReferences": (TermIdentifier(label="crossReference"), "multiple"),
    }

    def convert_to_api_term(self, term: Term, with_references=True) -> Dict:
        data = super().convert_to_api_term(term, with_references)

        if "addictoSubOntology" in data and data["addictoSubOntology"] is not None:
            lower_level_ontologies = [d.lower().strip() for d in data["addictoSubOntology"]]
            if "upper level" in lower_level_ontologies:
                lower_level_ontologies.remove("upper level")

            data["addictoSubOntology"] = lower_level_ontologies

        return data

    async def convert_from_api_term(self, data: Dict, with_references=True) -> Term:
        term = await super().convert_from_api_term(data, with_references)

        term.relations = [(r, bool(int(v))) if r.id is None and r.label in ["eCigO", "fuzzySet"] else (r, v) for r, v in
                          term.relations]

        return term

    def terms_equal(self, old: Term, new: Term) -> bool:
        ignore_if_not_exists = [
            "informalDefinition",
            "addictoSubOntology",
            "fuzzySet",
            "fuzzyExplanation",
            "crossReference",
            "eCigO"
        ]

        return self._terms_equal(ignore_if_not_exists, new, old)



import asyncio
from datetime import datetime
from typing import List

import aiohttp
from flask_github import GitHub
from flask_sqlalchemy import SQLAlchemy

from ose.model.ExcelOntology import ExcelOntology
from ose.model.ReleaseScript import ReleaseScript
from ose.model.Result import Result
from ose.release.ReleaseStep import ReleaseStep
from ose.release.common import order_sources
from .AddictOVocabService import AddictOVocabService
from ose.services.ConfigurationService import ConfigurationService


class AddictOVocabReleaseStep(ReleaseStep):
    _included_files: List[str]

    @classmethod
    def name(cls) -> str:
        return "ADDICTO_VOCAB"

    def __init__(self, db: SQLAlchemy, gh: GitHub, release_script: ReleaseScript, release_id: int, tmp: str,
                 config: ConfigurationService, *, included_files: List[str]):
        super().__init__(db, gh, release_script, release_id, tmp, config)

        self._included_files = included_files

    def run(self) -> bool:
        result = Result()
        sources = order_sources(
            dict([(k, f) for k, f in self._release_script.files.items() if k in self._included_files]))

        ontology = ExcelOntology("")
        external_ontology_result = self._load_externals_ontology()
        if not external_ontology_result.ok():
            self._set_release_result(external_ontology_result)
            return False

        self._raise_if_canceled()

        external = external_ontology_result.value

        ontology.import_other_excel_ontology(external)

        for i, (k, file) in enumerate(sources):
            for s in file.sources:
                if s.type == "classes":
                    result += ontology.add_terms_from_excel(s.file, self._local_name(s.file))
                elif s.type == "relations":
                    result += ontology.add_relations_from_excel(s.file, self._local_name(s.file))

                self._raise_if_canceled()

        ontology.resolve()
        self._raise_if_canceled()

        ontology.remove_duplicates()
        self._raise_if_canceled()

        externals = [self._local_name(self._release_script.external.target.file)]
        result += asyncio.run(self.run_service(ontology, externals))

        self._raise_if_canceled()

        self._set_release_result(result)
        return result.ok()

    async def run_service(self, ontology: ExcelOntology, externals: List[str]) -> Result[tuple]:
        async with aiohttp.ClientSession() as session:
            service = AddictOVocabService(self._config, session)
            return await service.update_api(
                ontology,
                externals,
                f"{datetime.utcnow().strftime('%B %Y')} Release",
                lambda step, total, msg: self._update_progress(position=(step, total), current_item=msg))



from importlib.metadata import version, PackageNotFoundError

try:
    __version__ = version("ose-plugin-addicto")
except PackageNotFoundError:
    __version__ = "unknown"

from ose.model.Plugin import Plugin, PluginComponent
from .AddictOVocabReleaseStep import AddictOVocabReleaseStep


AddictOPlugin = Plugin(
    id="org.bssofoundry.addicto",
    name="AddictO Plugin",
    version="0.1.0",
    description="Plugin for AddictO services and workflows.",
    contents=[
        AddictOVocabReleaseStep,
    ],
    components=[
        PluginComponent(
            step_name="ADDICTO_VOCAB",
            component_name="AddictOVocab",
        ),
    ],
    js_module="ose-plugin-addicto.js",
)
